﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mono.Cecil;
using Mono.Cecil.Cil;
using NETDeob.Engine.Utils;
using NETDeob.Misc;
using NETDeob.Misc.Structs__Enums___Interfaces.Deobfuscation;
using NETDeob.Misc.Structs__Enums___Interfaces.Tasks.Base;

namespace NETDeob.Deobfuscators.Generic
{
    public class AssemblyStripper : DeobfuscationTask
    {
        public AssemblyStripper(AssemblyDefinition asmDef)
            : base(asmDef)
        {
        }

        [DeobfuscationPhase(1, "Strip assembly from bad members")]
        public static bool Phase1()
        {
            return RemoveMembers();
        }

        private static bool RemoveMembers()
        {
            var tmp = new MarkedMember();

            var buf1 = new byte[] {1, 2, 3, 4, 5, 6, 7, 8, 9};
            var buf2 = new byte[8];

            Buffer.BlockCopy(buf1, 1, buf2, 0, buf1.Length - 2);

            try
            {
                foreach (var member in DeobfuscatorContext.MarkedMembers)
                {
                    switch (member.Type)
                    {
                        case MemberType.Type:
                            Logger.VLog("[Remove(Type)] " + (member.Member as TypeDefinition).Name);
                            AsmDef.MainModule.Types.Remove(member.Member as TypeDefinition);
                            break;

                        case MemberType.Method:
                            Logger.VLog("[Remove(Method)] " + (member.Member as MethodDefinition).Name);
                            (member.Member as MethodDefinition).DeclaringType.Methods.Remove(
                                member.Member as MethodDefinition);
                            break;

                        case MemberType.Field:
                            Logger.VLog("[Remove(Field)] " + (member.Member as FieldDefinition).Name);
                            (member.Member as FieldDefinition).DeclaringType.Fields.Remove(
                                member.Member as FieldDefinition);
                            break;

                        case MemberType.Property:
                            Logger.VLog("[Remove(Property)] " + (member.Member as PropertyDefinition).Name);
                            (member.Member as PropertyDefinition).DeclaringType.Properties.Remove(
                                member.Member as PropertyDefinition);
                            break;

                        case MemberType.Resource:
                            Logger.VLog("[Remove(Resource)] " + (member.Member as EmbeddedResource).Name);
                            AsmDef.MainModule.Resources.Remove(member.Member as EmbeddedResource);
                            break;

                        case MemberType.Attribute:
                            Logger.VLog("[Remove(Attribute)] " + (member.Member as CustomAttribute).AttributeType.Name);
                            (member.ParentMember as ModuleDefinition).CustomAttributes.Remove(
                                member.Member as CustomAttribute);
                            break;

                        case MemberType.Instruction:
                            Logger.VLog("[Remove(Instruction)] " + (member.Member as Instruction).OpCode);
                            (member.ParentMember as MethodDefinition).Body.Instructions.Remove(
                                member.Member as Instruction);
                            break;
                    }

                    tmp = member;
                }
            }
            catch (Exception e)
            {
                if (tmp == null)
                    return true;

                ThrowPhaseError("Failed to clean up member!", 0, false);
            }

            Logger.VSLog(string.Format("{0} members cleaned from assembly...", DeobfuscatorContext.MarkedMembers.Count));
            return true;
        }
    }
}
