﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using NETDeob.Misc;
using NETDeob._Console.Bases;

namespace NETDeob._Console.Utils
{
    class ArgumentParser
    {
        private List<Command> _commands = new List<Command>
                                             {
                                                 new CmdVerbose(),
                                                 new CmdHelp(),
                                                 new CmdOut()
                                             };

        public List<Command> ParsedCommands { get; private set; }

        private string[] _rawInput;

        public ArgumentParser(string[] rawInput)
        {
            if(!File.Exists(rawInput[0])){
                Console.WriteLine("Could not find file {0}!", rawInput[0]);
                Console.ReadLine();
                Environment.Exit(-1);
            }

            DeobfuscatorContext.InPath = rawInput[0];

            _rawInput = rawInput.FromIndex(1);
            ParsedCommands = new List<Command>();
        }

        public bool ParseRawInput()
        {
            foreach(var raw in _rawInput)
            {
                var cmd = _commands.FirstOrDefault(c => c.RawCommand.Substring(0, 4) == raw.ToLower().Substring(0, 4));

                if (cmd == null){
                    DisplayInvalid(raw);
                    return false;
                }

                if (cmd is CmdHelp){
                    DisplayHelp();
                    return false;
                }

                if (cmd is CmdOut)
                {
                    if (raw != _rawInput.Last())
                    {
                        DisplayInvalid(raw);
                        return false;
                    }
                }

                cmd.UserInput = raw.Substring(4);
                ParsedCommands.Add(cmd);
            }

            return true;
        }

        private void DisplayInvalid(string raw)
        {
            Console.WriteLine("Invalid command: {0}!\n\nType NETDeob.exe /help for more information.", raw);
        }

        private void DisplayHelp()
        {
            Console.WriteLine("Usage:\n\nNETDeob.exe [filename in] [/commands] [/out:filename out]\n\nHelp:\n");

            foreach(var cmd in _commands)
            {
                Console.WriteLine("{0}: {1}", cmd.RawCommand, cmd.Description);
            }
        }
    }
}
